//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;
    using System.Runtime.Serialization;
    using System.ServiceModel;
    using System.ServiceModel.Description;
    using System.ServiceModel.Dispatcher;

    public class QueryStringFormatterBehavior : IEndpointBehavior
    {
        public void AddBindingParameters(ServiceEndpoint serviceEndpoint, System.ServiceModel.Channels.BindingParameterCollection bindingParameters)
        {
        }

        void IEndpointBehavior.ApplyClientBehavior(ServiceEndpoint serviceEndpoint, ClientRuntime behavior)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "QueryStringFormatterBehavior.ApplyClientBehavior");
            IOperationBehavior innerFormatter;
            foreach (OperationDescription operationDescription in serviceEndpoint.Contract.Operations)
            {
                innerFormatter = GetInnerFormatter(operationDescription);

                foreach (ClientOperation operationRuntime in behavior.Operations)
                {
                    if (operationDescription.Name == operationRuntime.Name)
                    {
                        innerFormatter.ApplyClientBehavior(operationDescription, operationRuntime);
                        operationRuntime.Formatter = new QueryStringFormatter(operationRuntime.SyncMethod.GetParameters(), operationRuntime.Formatter, operationRuntime.Action, serviceEndpoint.Address);
                    }
                }
                TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Successfully applied QueryStringFormatter", "QueryStringFormatterBehavior.ApplyClientBehavior");
            }
        }

        public void ApplyDispatchBehavior(ServiceEndpoint serviceEndpoint, EndpointDispatcher endpointDispatcher)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "QueryStringFormatterBehavior.ApplyDispatchBehavior");
            IOperationBehavior innerFormatter;
            foreach (OperationDescription operationDescription in serviceEndpoint.Contract.Operations)
            {
                innerFormatter=GetInnerFormatter(operationDescription);
                
                foreach (DispatchOperation operationRuntime in endpointDispatcher.DispatchRuntime.Operations)
                {
                    if (operationDescription.Name == operationRuntime.Name)
                    {
                        innerFormatter.ApplyDispatchBehavior(operationDescription, operationRuntime);
                        operationRuntime.Formatter = new QueryStringFormatter(operationDescription.SyncMethod.GetParameters(), operationRuntime.Formatter);
                    }
                }
                TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Successfully applied QueryStringFormatter", "QueryStringFormatterBehavior.ApplyDispatchBehavior");
            }
        }

        public void Validate(ServiceEndpoint serviceEndpoint)
        {
            
        }
        IOperationBehavior GetInnerFormatter(OperationDescription operationDescription)
        {
            //remove the DC or XmlSerializer behaviors
            IOperationBehavior innerFormatter=operationDescription.Behaviors.Remove<DataContractSerializerOperationBehavior>();
            if (innerFormatter == null)
            {
                //look for XmlSer behavior
                innerFormatter = operationDescription.Behaviors.Remove<XmlSerializerOperationBehavior>();
                if (innerFormatter == null)
                {
                    //look for marker behavior
                    InnerFormatterMarkerBehavior markerBehavior = operationDescription.Behaviors.Find<InnerFormatterMarkerBehavior>();
                    if (markerBehavior != null)
                    {
                        innerFormatter = markerBehavior.InnerFormatter;
                        return innerFormatter; 
                    }
                    else
                    {
                        //didn't find DC or XmlSer or the marker behavior
                        TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Error, "Could not find DataContractFormatter or XmlSerializer on the contract", "QueryStringFormatterBehavior.GetInnerFormatter");
                        throw new InvalidOperationException("Could not find DataContractFormatter or XmlSerializer on the contract");

                    }

                }
            }
            //remember what the innerFormatter was
            InnerFormatterMarkerBehavior newmarkerBehavior = new InnerFormatterMarkerBehavior();
            newmarkerBehavior.InnerFormatter = innerFormatter;
            operationDescription.Behaviors.Add(newmarkerBehavior);
            return innerFormatter;
        }
    }
    //This dummy behavior is just a marker
    //it's used to tell the QueryStringFormatterBehavior
    //which formatter to use internally
    class InnerFormatterMarkerBehavior : IOperationBehavior
    {
        IOperationBehavior innerFormatter;
        internal IOperationBehavior InnerFormatter
        {
            get { return innerFormatter; }
            set { innerFormatter = value; }
        }
        #region IOperationBehavior Members

        public void AddBindingParameters(OperationDescription description, System.ServiceModel.Channels.BindingParameterCollection parameters)
        {
            if (innerFormatter != null)
            {
                innerFormatter.AddBindingParameters(description, parameters);
            }
            
        }

        public void ApplyClientBehavior(OperationDescription description, ClientOperation proxy)
        {
            if (innerFormatter != null && proxy.Formatter == null)
            {
                //no formatter has been applied yet
                innerFormatter.ApplyClientBehavior(description, proxy);
                
            }
        }

        public void ApplyDispatchBehavior(OperationDescription description, DispatchOperation dispatch)
        {
            if (innerFormatter != null && dispatch.Formatter == null)
            {
                //no formatter has been applied yet
                innerFormatter.ApplyDispatchBehavior(description, dispatch);
            }            
        }

        public void Validate(OperationDescription description)
        {
            if (innerFormatter != null)
            {
                innerFormatter.Validate(description);
            }
        }

        #endregion
    }
}
