//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.ServiceModel.Dispatcher;
    using System.Text;
    using System.Web;
    using System.ComponentModel;
    using System.Collections.Specialized;
    
    class QueryStringFormatter : IClientMessageFormatter, IDispatchMessageFormatter //IClientFormatter, IDispatchFormatter
    {
       

        string action;
        EndpointAddress address;
        IClientMessageFormatter innerClientFormatter;
        IDispatchMessageFormatter innerDispatchFormatter;
        ParameterInfo[] parameterInfos;

        public QueryStringFormatter(ParameterInfo[] parameterInfos, IClientMessageFormatter innerClientFormatter, string action, EndpointAddress address)
        {
            if (parameterInfos == null)
                throw new ArgumentNullException("parameterInfos");

            if (innerClientFormatter == null)
                throw new ArgumentNullException("innerClientFormatter");

            if (action == null)
                throw new ArgumentNullException("action");

            if (address == null)
                throw new ArgumentNullException("address");
            QueryStringFormatter.ValidateSupportedType(parameterInfos);
            this.action = action;
            this.address = address;
            this.innerClientFormatter = innerClientFormatter;
            this.parameterInfos = parameterInfos;
        }

        public QueryStringFormatter(ParameterInfo[] parameterInfos, IDispatchMessageFormatter innerDispatchFormatter)
        {
            if (parameterInfos == null)
                throw new ArgumentNullException("parameterInfos");

            if (innerDispatchFormatter == null)
                throw new ArgumentNullException("innerDispatchFormatter");

            QueryStringFormatter.ValidateSupportedType(parameterInfos);
            this.innerDispatchFormatter = innerDispatchFormatter;
            this.parameterInfos = parameterInfos;
        }

        Message IClientMessageFormatter.SerializeRequest(MessageVersion messageVersion, object[] parameters)
        {
            Message message = Message.CreateMessage(messageVersion, action);
            
            address.ApplyTo(message);
            UriBuilder builder = new UriBuilder(message.Headers.To);
            builder.Query = EncodeParameters(parameterInfos, parameters);
            message.Headers.To = builder.Uri;
            message.Properties.Via = builder.Uri;

            return message;
        }

        object IClientMessageFormatter.DeserializeReply(Message message, object[] parameters)
        {
            return this.innerClientFormatter.DeserializeReply(message, parameters);
        }

        void IDispatchMessageFormatter.DeserializeRequest(Message message, object[] parameters)
        {
            HttpMessageContext context=HttpMessageContext.GetContext(message);
            if (context.QueryString.Count == 0)
            {
                //no query string
                TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "No query string found in HttpMessageContext", "QueryStringFormatter.DeserializeRequest");
                return;
            }
            DecodeParameters(parameterInfos, context.QueryString, parameters);
        }

        Message IDispatchMessageFormatter.SerializeReply(MessageVersion messageVersion, object[] parameters, object result)
        {
            return this.innerDispatchFormatter.SerializeReply(messageVersion, parameters, result);
        }

        string EncodeParameters(ParameterInfo[] parameterInfos, object[] parameters)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "QueryStringFormatter.EncodeParameters");
            if (parameterInfos.Length != parameters.Length)
                throw new ArgumentException("parameterInfos and parameters must conain the same number of items");

            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < parameters.Length; i++)
            {
                if (builder.Length > 0)
                    builder.Append('&');

                builder.Append(HttpUtility.UrlEncode(parameterInfos[i].Name));
                builder.Append('=');
                builder.Append(HttpUtility.UrlEncode(parameters[i].ToString()));
            }
            TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Successfully encoded parameters to query string", "QueryStringFormatter.EncodeParameters");
            return builder.ToString();
        }

        void DecodeParameters(ParameterInfo[] parameterInfos, Dictionary<string, string> dictionary, object[] parameters)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "QueryStringFormatter.DecodeParameters");

            if (parameterInfos.Length != parameters.Length)
                throw new ArgumentException("parameterInfos and parameters must conain the same number of items");

            
            for (int i = 0; i < parameters.Length; i++)
            {
                string value;
                if (dictionary.TryGetValue(parameterInfos[i].Name.ToLowerInvariant(), out value))
                {
                    TypeConverter converter = TypeDescriptor.GetConverter(parameterInfos[i].ParameterType);
                    object typedValue = converter.ConvertFromString(value);
                    parameters[i] = typedValue;
                }
                else
                {
                    //ignore missing parameters
                }
            }
            TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Successfully decoded parameters from query string", "QueryStringFormatter.DecodeParameters");
        }
        internal static void ValidateSupportedType(ParameterInfo[] parameterInfos)
        {
            Type stringtype = typeof(String);
            foreach (ParameterInfo param in parameterInfos)
            {
                TypeConverter converter = TypeDescriptor.GetConverter(param.ParameterType);
                if (!converter.CanConvertFrom(stringtype))
                {
                    Exception ex=new Exception(String.Format("QueryStringFormatter does not support parameters of type {0}. Only simple types are supported.", param.ParameterType));
                    TraceHelper.WriteTraceData(System.Diagnostics.TraceEventType.Error, 0, "Unsupported data types", ex.Message, "QueryStringFormatter.ValidateSupportedType", ex);
                }
            }

        }
    }
}
